'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var uuid = require('node-uuid');

// Error constants
var CONFIG_REQUIRED = 'Configuration Object Required';
var PROCESS_REQUIRED = 'required paramenter [process] must be a function';
var SOURCE_REQUIRED = 'Source is required to be either a function or an array';
var TYPE_PROCEED_ON_ERROR = 'parameter proceedOnError must be a boolean';
var TYPE_EVENT_HANDLER = 'Event handlers must be functions';

var JobQueuer = function () {
  function JobQueuer(config) {
    _classCallCheck(this, JobQueuer);

    if (!config) throw new Error(CONFIG_REQUIRED);
    if (!config.process || typeof config.process !== 'function') throw new Error(PROCESS_REQUIRED);
    if (!config.source || typeof config.source !== 'function' && !Array.isArray(config.source)) throw new Error(SOURCE_REQUIRED);
    if (config.proceedOnError && !_typeof(config.proceedOnError) === 'boolean') throw new Error(TYPE_PROCEED_ON_ERROR);
    this.events = {};
    this.maxProceses = config.maxProceses || 1;
    this.process = config.process;
    this.source = config.source;
    this.proceedOnError = config.proceedOnError || false;
    this.sourceType = Array.isArray(this.source) ? 'array' : 'function';
    this.running = {};
    this.jobsFinished = 0;
    this.jobErrors = 0;
    this.fillingJobs = false;
    this.status = 'stoped';
  }

  _createClass(JobQueuer, [{
    key: 'on',
    value: function on(event, handler) {
      this.events[event] = handler;
    }
  }, {
    key: 'emit',
    value: function emit(event, payload) {
      if (this.debug && console) console.log('[' + new Date() + '][' + event + ']');
      if (this.events[event]) this.events[event](payload);
    }
  }, {
    key: 'start',
    value: function start() {
      this.status = 'running';
      this.startTime = new Date();
      this.emit('start', {
        startTime: this.startTime,
        maxProceses: this.maxProceses,
        proceedOnError: this.proceedOnError,
        sourceType: this.sourceType,
        status: this.status
      });
      this.fillJobs();
    }
  }, {
    key: 'runningJobsCount',
    value: function runningJobsCount() {
      return Object.keys(this.running).length;
    }
  }, {
    key: 'runJob',
    value: function runJob(jobPromise) {
      var _this = this;

      var jobId = uuid.v4();
      this.emit('jobRun', jobId);
      this.running[jobId] = jobPromise;
      var next = function next() {
        var self = _this;
        var jobToDelete = _this.running[jobId];
        delete _this.running[jobId];
        var runningCount = Object.keys(_this.running).length;
        if (!runningCount) {
          _this.status = 'finished';
          return _this.emit('processFinish', {
            startTime: _this.startTime,
            endTime: new Date(),
            processed: _this.jobsFinished,
            errors: _this.jobErrors,
            status: _this.status
          });
        }
        _this.fillJobs();
      };

      var jobStartTime = new Date();
      jobPromise.then(function (result) {
        var jobEndTime = new Date();
        _this.emit('jobFinnish', {
          jobId: jobId,
          jobStartTime: jobStartTime,
          jobEndTime: jobEndTime,
          result: result,
          jobsRunning: Object.keys(_this.running).length
        });
        _this.jobsFinished++;
        next();
      }).catch(function (e) {
        _this.emit('error', e);
        _this.jobErrors++;
        next();
      });
    }
  }, {
    key: 'fillJobs',
    value: function fillJobs() {
      var _this2 = this;

      if (this.fillingJobs) return;
      this.fillingJobs = true;

      var resolveJobValue = function resolveJobValue(jobValue, resolve, reject) {
        if (jobValue) {
          var jobPromise = _this2.process(jobValue, function (err, value) {
            if (err) return reject(err);
            resolve(value);
          });
          if (jobPromise) {
            if (typeof jobPromise.then === 'function') return jobPromise.then(resolve);
            resolve(jobPromise);
          }
        } else {
          resolve();
          _this2.status = 'empty';
        }
      };

      while (Object.keys(this.running).length < this.maxProceses && this.status === 'running' && (this.sourceType === 'array' && this.source.length || this.sourceType !== 'array')) {
        this.emit('jobFetch', {
          jobsRunning: Object.keys(this.running).length
        });
        this.runJob(new Promise(function (resolve, reject) {
          var item = void 0;
          if (_this2.sourceType === 'array') {
            item = _this2.source.pop();
          } else {
            item = _this2.source(function (err, jobValue) {
              if (err) return reject(err);
              resolveJobValue(jobValue, resolve, reject);
            });
          }

          if (item && item.then && typeof item.then === 'function') {
            item.then(function (jobValue) {
              resolveJobValue(jobValue, resolve, reject);
            }).catch(reject);
          } else {
            if (item) {
              resolveJobValue(item, resolve, reject);
            }
          }
        }));
      }
      this.fillingJobs = false;
    }
  }]);

  return JobQueuer;
}();

module.exports = JobQueuer;